'Extended CheckBoxList

'1) Adds optional "All" and "None" checkboxes to the list and via the
'   addition of an "onClick" attribute to each item in the checkboxlist calls a client-side
'   Javascript function each time a checkbox in the list is clicked to both handle setting all
'   or none of the checkboxes and updating the all or none checkbox as the count of selected checkboxes
'   change.
'2) A limit on the maximum number of checkboxes that can be selected can be
'   specified. When this limit is exceeded, an optional pop-up alert can be shown and the attempted
'   check ignored.
'3) Overrides the ClearSelection method to properly set the None checkbox if it is enabled.
'4) Implements a SetAll method to set all checkboxes (with the exception of the None checkbox if it is enabled).
'5) Multiple checkboxlists containing the same number of checkboxes can be given a Groupname in which
'   case only one of the corresponding checkboxes in the grouped checkbox lists may be selected.
'
'Written By William Severance, WESNet Designs
'
'Date Last Modified: April 20, 2006
'
'Website: http://www.wesnetdesigns.com
'E-Mail: imagemaker@wesnetdesigns.com

'Disclaimer: This software is provided without charge "AS IS" with no warranties of any kind. The entire risk
'arising out of the use or performance of the software is with you.

Imports System
Imports System.ComponentModel
Imports System.Text
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls

<Assembly: WebResource("WESNet.WebControls.ExtendedCheckBoxList.js", "application/x-javascript")> 

Public Class ExtendedCheckBoxList
    Inherits CheckBoxList

#Region "Public Enums"

    'Specify placement of "All" and/or "None" checkboxes at beginning of list or end of list
    Public Enum SpecialCheckboxPlacement As Integer
        BeginningOfList = 0
        EndOfList
    End Enum
#End Region

#Region "Public Properties"

    <Browsable(True), Category("Behavior"), DefaultValue(False), _
     Description("When set to true an 'All' checkbox is inserted at the beginning of the list")> _
    Public Property EnableAllCheckBox() As Boolean
        Get
            If ViewState("EnableAll") Is Nothing Then
                Return False
            Else
                Return CType(ViewState("EnableAll"), Boolean)
            End If
        End Get
        Set(ByVal value As Boolean)
            ViewState("EnableAll") = value
        End Set
    End Property

    <Browsable(True), Category("Appearance"), DefaultValue("All"), _
     Description("Text to display beside the 'All' checkbox")> _
    Public Property AllCheckBoxText() As String
        Get
            If ViewState("AllText") Is Nothing Then
                Return "All"
            Else
                Return CType(ViewState("AllText"), String)
            End If
        End Get
        Set(ByVal value As String)
            ViewState("AllText") = value
        End Set
    End Property

    <Browsable(True), Category("Behavior"), DefaultValue(False), _
     Description("When set to true a 'None' checkbox is inserted at the beginning of the list")> _
    Public Property EnableNoneCheckBox() As Boolean
        Get
            If ViewState("EnableNone") Is Nothing Then
                Return False
            Else
                Return CType(ViewState("EnableNone"), Boolean)
            End If
        End Get
        Set(ByVal value As Boolean)
            ViewState("EnableNone") = value
        End Set
    End Property

    <Browsable(True), Category("Appearance"), DefaultValue("None"), _
     Description("Text to display beside the 'None' checkbox")> _
    Public Property NoneCheckBoxText() As String
        Get
            If ViewState("NoneText") Is Nothing Then
                Return "None"
            Else
                Return CType(ViewState("NoneText"), String)
            End If
        End Get
        Set(ByVal value As String)
            ViewState("NoneText") = value
        End Set
    End Property

    <Browsable(True), Category("Layout"), DefaultValue(GetType(SpecialCheckboxPlacement), "BeginningOfList"), _
     Description("Selects placement of 'All' and/or 'None' checkboxes at beginning or end of list")> _
     Public Property Placement() As SpecialCheckboxPlacement
        Get
            If ViewState("Placement") Is Nothing Then
                Return SpecialCheckboxPlacement.BeginningOfList
            Else
                Return CType(ViewState("Placement"), SpecialCheckboxPlacement)
            End If
        End Get
        Set(ByVal value As SpecialCheckboxPlacement)
            ViewState("Placement") = value
        End Set
    End Property

    <Browsable(True), Category("Behavior"), DefaultValue(999), _
     Description("If set controls the maximum number of checkboxes that can be checked in the list")> _
    Public Property MaxChecks() As Integer
        Get
            If ViewState("MaxChecks") Is Nothing Then
                Return 999
            Else
                Return CType(ViewState("MaxChecks"), Integer)
            End If
        End Get
        Set(ByVal value As Integer)
            ViewState("MaxChecks") = value
        End Set
    End Property

    <Browsable(True), Category("Behavior"), DefaultValue(False), _
     Description("Set to True to display MaxChecksErrorMessage when MaxChecks is exceeded")> _
    Public Property EnableMaxChecksAlert() As Boolean
        Get
            If ViewState("EnableMaxChecksAlert") Is Nothing Then
                Return False
            Else
                Return CType(ViewState("EnableMaxChecksAlert"), Boolean)
            End If
        End Get
        Set(ByVal value As Boolean)
            ViewState("EnableMaxChecksAlert") = value
        End Set
    End Property

    <Browsable(True), Category("Appearance"), _
     Description("Message to be displayed when MaxChecks is exceeded and EnableMaxChecksAlert is True")> _
    Public Property MaxChecksErrorMessage() As String
        Get
            If ViewState("MaxChecksErrorMessage") Is Nothing Then
                Return "Only " & MaxChecks.ToString & " check boxes may be selected!"
            Else
                Return CType(ViewState("MaxChecksErrorMessage"), String)
            End If
        End Get
        Set(ByVal value As String)
            ViewState("MaxChecksErrorMessage") = value
        End Set
    End Property

    <Browsable(True), Category("Behavior"), DefaultValue(""), _
     Description("Group name for use with multiple checkboxlists having the same elements")> _
    Public Property Groupname() As String
        Get
            If ViewState("Groupname") Is Nothing Then
                Return String.Empty
            Else
                Return CType(ViewState("Groupname"), String)
            End If
        End Get
        Set(ByVal value As String)
            ViewState("Groupname") = value
        End Set
    End Property

#End Region

#Region "Public Methods"

    'Clears selection of all checkboxes and checks the "None" checkbox if it is enabled
    Public Overrides Sub ClearSelection()
        For Each li As ListItem In Items
            li.Selected = (EnableNoneCheckBox AndAlso (li.Text = NoneCheckBoxText))
        Next
    End Sub

    'Sets selection of all checkboxes and clears selection of "None" checkbox if it is enabled
    Public Sub SelectAll()
        For Each li As ListItem In Items
            li.Selected = Not (EnableNoneCheckBox AndAlso (li.Text = NoneCheckBoxText))
        Next
    End Sub

    'Returns a collection of the selected items excluding the "None" and "All" checkboxes if
    'enabled
    Public Function SelectedItems() As ListItemCollection
        Dim lc As New ListItemCollection
        For Each li As ListItem In Items
            If Not ((EnableNoneCheckBox AndAlso li.Text = NoneCheckBoxText) _
                  OrElse (EnableAllCheckBox AndAlso li.Text = AllCheckBoxText)) _
                    And li.Selected Then lc.Add(li)
        Next
        Return lc
    End Function

    'Returns an array of Strings containing the value of the selected items excluding the
    '"None" and "All" checkboxes if enabled.
    Public Function SelectedValues() As String()
        Dim values As New ArrayList
        Dim i As Integer = 0
        For Each li As ListItem In Items
            If Not ((EnableNoneCheckBox AndAlso li.Text = NoneCheckBoxText) _
                             OrElse (EnableAllCheckBox AndAlso li.Text = AllCheckBoxText)) _
                               And li.Selected Then values.Add(li.Value)
        Next
        Return CType(values.ToArray(GetType(System.String)), String())
    End Function

    'Returns true if all items (not including the special "All" and "None" checkboxes) are
    'selected
    Public Function AllSelected() As Boolean
        Dim ItemsSelected As Integer = 0
        Dim TotalItems As Integer = 0
        For Each li As ListItem In Items
            If Not ((EnableNoneCheckBox AndAlso li.Text = NoneCheckBoxText) _
                  OrElse (EnableAllCheckBox AndAlso li.Text = AllCheckBoxText)) Then
                TotalItems += 1
                If li.Selected Then ItemsSelected += 1
            End If
        Next
        Return ItemsSelected = TotalItems
    End Function

    'Returns true if no items (not including the special "All" and "None" checkboxes) are
    'selected
    Public Function NoneSelected() As Boolean
        Dim ItemsSelected As Integer = 0
        Dim TotalItems As Integer = 0
        For Each li As ListItem In Items
            If Not ((EnableNoneCheckBox AndAlso li.Text = NoneCheckBoxText) _
                  OrElse (EnableAllCheckBox AndAlso li.Text = AllCheckBoxText)) _
                     And li.Selected Then ItemsSelected += 1
        Next
        Return ItemsSelected = 0
    End Function

#End Region

#Region"Private Methods 

    Private Sub ExtendedCheckBoxList_PreRender(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.PreRender

        Dim AllListItem As ListItem = Nothing
        Dim NoneListItem As ListItem = Nothing
        Dim InsertionIndex As Integer = 0

        Page.ClientScript.RegisterClientScriptResource(GetType(ExtendedCheckBoxList), "WESNet.WebControls.ExtendedCheckBoxList.js")

        If Placement = SpecialCheckboxPlacement.EndOfList Then InsertionIndex = Items.Count

        If Page.IsPostBack Then
            If EnableAllCheckBox OrElse EnableNoneCheckBox Then
                For Each li As ListItem In Items
                    If li.Text = AllCheckBoxText Then
                        AllListItem = li
                    ElseIf li.Text = NoneCheckBoxText Then
                        NoneListItem = li
                    End If
                Next
            End If
        Else
            If EnableAllCheckBox Then
                AllListItem = New ListItem(AllCheckBoxText)
                Items.Insert(InsertionIndex, AllListItem)
                InsertionIndex += 1
            End If

            If EnableNoneCheckBox Then
                NoneListItem = New ListItem(NoneCheckBoxText)
                Items.Insert(InsertionIndex, NoneListItem)
            End If
        End If


        If MaxChecks <> 999 Then
            Me.Attributes.Add("maxchecks", MaxChecks.ToString)
            If EnableMaxChecksAlert Then Me.Attributes.Add("maxcheckserrormessage", MaxChecksErrorMessage)
        End If

        If EnableAllCheckBox Then Me.Attributes.Add("alltext", AllCheckBoxText)
        If EnableNoneCheckBox Then Me.Attributes.Add("nonetext", NoneCheckBoxText)

        If Groupname <> String.Empty Then Me.Attributes.Add("groupname", Groupname)

        'Note: Although the ASP.NET 2.0 ListItem properly renders added Attributes (unlike ASP.NET 1.1),
        'they are still not persisted across postbacks and must be added each time before the CheckBoxList renders

        Dim ItemsSelected As Integer = 0

        For Each li As ListItem In Items
            li.Attributes.Add("onClick", "javascript: ExtendedCheckBoxList_Click(this);")
            If ((Not AllListItem Is Nothing) AndAlso (Not li Is AllListItem)) _
                 Or ((Not NoneListItem Is Nothing) AndAlso (Not li Is NoneListItem)) Then
                If li.Selected Then ItemsSelected += 1
            End If
        Next

        Dim TotalItems As Integer = Items.Count

        If Not NoneListItem Is Nothing Then
            TotalItems -= 1
            If ItemsSelected = 0 Then
                NoneListItem.Selected = True
                Exit Sub
            End If
        End If

        If Not AllListItem Is Nothing Then
            TotalItems -= 1
            If ItemsSelected = TotalItems Then AllListItem.Selected = True
        End If

    End Sub
#End Region

End Class
